#include <interfacePaCO_Basic_Example_server.h>
#include <mpi.h>
#include <com/paco_mpi.h>

#ifndef PADICO
#include <thread/paco_omni.h>
#include <CORBA.h>
#include <Naming.hh>
#else
#include <thread/paco_marcel.h>
#include <Padico/CORBA.h>
#include <Padico/COSNaming.h>
#include <Padico/boot-notifier.h>
#include <Padico/PadicoTM.h>
#endif

unsigned nserver;

void usage()
{
    fprintf(stderr,"Usage: Proxy <server size (1-100)>.\n");
}

int main(int argc, char **argv)
{
    // Init
    cout << "[Proxy] Initialisation du proxy" << endl;
    MPI_Init(&argc, &argv);

    // Init de CORBA
#ifndef PADICO
    CORBA::ORB_var orb = CORBA::ORB_init(argc, argv);
#else
    //Padico::ORB_ptr _orb = Padico::ORB_init();
    CORBA::ORB_ptr orb = Padico::the_orb;
#endif

    // Init PACO
    paco_fabrique_manager* pfm = paco_getFabriqueManager();

#ifdef PADICO
    padico_boot_notify_t n = padico_boot_notify_init(&argc, argv);
#endif

    // CommandLine Parameter
    if (argc<1)
    {
        cerr << "bad #arg value: " << argc << endl;
        for(int i=0;i<argc;i++)
            cerr << i << ": " << argv[i] << endl;
        usage();
        goto err;
    }
    sscanf(argv[1],"%d",&nserver);
    if ((nserver<=0)||(nserver>100))
    {
        cerr << "bad nserver value: " << nserver << endl;
        usage();
        goto err;
    }
    {
        CORBA::Object_var obj = orb->resolve_initial_references("NameService");
        CosNaming::NamingContextExt_var nc =
            CosNaming::NamingContextExt::_narrow(obj);

        obj = orb->resolve_initial_references("RootPOA");
        PortableServer::POA_var poa = PortableServer::POA::_narrow(obj);

        // Creation du proxy
        cout << "[Proxy] Creating proxy" << endl;
        Basic::Example_proxy_impl * proxy = new Basic::Example_proxy_impl(orb);

	// Contexte global
	PaCO_operation * global_ptr =  proxy->getContext("global_paco_context");
	
#ifndef PADICO
	pfm->register_thread("mythread", new paco_omni_fabrique());
#else
	pfm->register_thread("mythread", new paco_marcel_fabrique());
#endif
	global_ptr->setLibThread("mythread");
	global_ptr->setTypeClient(true);

	pfm->register_com("mpi", new paco_mpi_fabrique());
	MPI_Comm group = MPI_COMM_WORLD;
	global_ptr->setLibCom("mpi",&group);

	PaCO::PacoTopology_t client_topo;
	client_topo.total = 1;
	global_ptr->setClientTopo(client_topo);

	PaCO::PacoTopology_t serveur_topo;
	serveur_topo.total = nserver;
	proxy->setTopo(serveur_topo);

        PortableServer::ObjectId_var proxyid = poa->activate_object(proxy);
        obj = proxy->POA_Basic::Example::_this();
        CosNaming::Name * name = nc->to_name("ParallelServeur");
        nc->rebind(*name,obj);

        // Fin Init
        cout << "[Proxy] Waiting requests" << endl;
        PortableServer::POAManager_var pman = poa->the_POAManager();
        pman->activate();

#ifndef PADICO

        orb->run();
#else

        {
	  padico_boot_notify(n, orb, nc);
            marcel_mutex_t m;
            marcel_mutex_init(&m,0);
            marcel_mutex_lock(&m);
            marcel_mutex_lock(&m);
        }
#endif
        // Fin
        //  cout << "[Proxy] Fin du serveur" << endl;
        //  orb->destroy();
    }
err:
    MPI_Finalize();
    cout << "[Proxy] Fin du proxy" << endl;
}

