#include<string>
#include <CORBA.h>
#include <Naming.hh>
#include <thread/paco_omni.h>
#include <interfacePaCO.h>
#include <interfacePaCO_Basic_Example_client.h>
#include <com/paco_dummy.h>
#include <stdio.h>
#include <GaBro.h>
#include <comScheduling/paco_direct_comScheduling.h>

static int myid = 0;
static int numprocs = 1;

static Basic::PaCO_Example * serveur;


// Loading librairies into the fabrics
void init() 
{
  paco_fabrique_manager* pfm = paco_getFabriqueManager();
  cerr << "[client] distributed lib step"  << endl;
  pfm->register_distribution("GaBro", new GaBro_fab());
  cerr << "[client] communication lib step"  << endl;
  pfm->register_com("dummy", new paco_dummy_fabrique());
  cerr << "[client] thread lib step"  << endl;  
#ifndef PADICO
  pfm->register_thread("omni", new paco_omni_fabrique());
#else
  pfm->register_thread("marcel", new paco_marcel_fabrique());
#endif
  cerr << "[client] scheduling lib step"  << endl;
  pfm->register_comScheduling("direct", new paco_direct_fabrique());
}


void test() 
{
  // Init CORBA data
  cerr << "[client] data creation" << endl;
  int len1 = 10;
  int len2 = 5;

  Basic::Example::lseq data1;
  data1.length(len1);
  for (unsigned i=0; i< len1; i++) 
    {
      data1[i] = i + i;
    }
  
  Basic::Example::lseq data2;
  data2.length(len2);
  for (unsigned i=0; i< len2; i++) 
    {
      data2[i] = i * i;
    }
  
  // Init PaCO++ layers
  
  // global context
  cerr << "[client] init global context" << endl;
  PaCO_operation * global_ptr = serveur->getContext("global_paco_context");
  global_ptr->setLibCom("dummy",NULL);
  global_ptr->setLibThread("omni");
  global_ptr->setTypeClient(true);
  PaCO::PacoTopology_t topo;
  topo.total = numprocs;
  global_ptr->setClientTopo(topo);
  global_ptr->setLibComScheduling("direct");

  // work_in context
  cerr << "[client] init work_in context" << endl;
  PaCO_operation * method_ptr = serveur->getContext("work_in");
  method_ptr->setDisLibArg(0,"GaBro","in");
  GaBro * dislib = (GaBro *) method_ptr->libraries_in[0];
  dislib->setNodePos(0);
  

  // End init PaCO++ layers
  cerr << "[client] final init" << endl;
  global_ptr->init(myid, numprocs);

  // data1
  dislib->setEltSize(sizeof(long));
  dislib->setNodeNbElt(len1);
  dislib->setTotalNbElt(len1);
  method_ptr->useCommunicationMemoryId(1);
  // Send data
  cerr << "[client] sending data1" << endl;
  serveur->work_in(data1);
  
  // data2
  dislib->setNodeNbElt(len2);
  dislib->setTotalNbElt(len2);
  method_ptr->useCommunicationMemoryId(2);
  // Send data
  cerr << "[client] sending data2" << endl;
  serveur->work_in(data2);

  // data1
  method_ptr->useCommunicationMemoryId(1);
  // Send data
  cerr << "[client] sending data1" << endl;
  serveur->work_in(data1);
}

int main(int argc, char **argv)
{
    // CORBA Init part
    cerr << "[Client] CORBA init" << endl;
#ifndef PADICO
    CORBA::ORB_var orb = CORBA::ORB_init(argc, argv);
#else
    Padico::ORB_ptr _orb = Padico::ORB_init();
    CORBA::ORB_ptr orb = Padico::the_orb;
#endif
    CORBA::Object_var obj = orb->resolve_initial_references("NameService");
    CosNaming::NamingContextExt_var nc =
        CosNaming::NamingContextExt::_narrow(obj);
    cerr << "[client] get the server" << endl;
    obj = nc->resolve_str("ParallelServeur");
    cerr << "[client] PaCO_narrow !!!" << endl;
    serveur = (Basic::PaCO_Example *) Basic::PaCO_Example::PaCO_narrow(obj, orb);

    init();

    // Test
    test();

    // End
    orb->destroy();
}
