#ifndef PADICO
#include <CORBA.h>
#include <Naming.hh>
#include <thread/paco_omni.h>
#else
#include <Padico/CORBA.h>
#include <Padico/COSNaming.h>
#include <thread/paco_marcel.h>
#endif
#include <iostream.h>
#include <servant_impl.h>
#include <stdio.h>
#include <com/paco_dummy.h>

#include <GaBro.h>

int main(int argc, char **argv)
{

    int myid = 0;
    int numprocs = 1;

    cerr<< "[Server] CORBA init" << endl;
#ifndef PADICO
    CORBA::ORB_ptr orb = CORBA::ORB_init(argc, argv);
#else
    Padico::ORB_ptr _orb = Padico::ORB_init();
    CORBA::ORB_ptr orb = Padico::the_orb;
#endif

    CORBA::Object_var obj = orb->resolve_initial_references("NameService");
    CosNaming::NamingContextExt_var nc = CosNaming::NamingContextExt::_narrow(obj);
    obj = orb->resolve_initial_references("RootPOA");
    PortableServer::POA_var root_poa = PortableServer::POA::_narrow(obj);
    PortableServer::POAManager_var pman = root_poa->the_POAManager();

    cerr << "[Server] finding proxy" << endl;
    obj = nc->resolve_str("ParallelServeur");

    cerr << "[Server] Creating servant" << endl;
    char * ior = orb->object_to_string(obj);
    example_serv_impl * servant = new example_serv_impl(orb,ior);
    
    paco_fabrique_manager * pfm = paco_getFabriqueManager();
    pfm->register_distribution("GaBro", new GaBro_fab());
    cerr << "[client] communication lib step"  << endl;
    pfm->register_com("dummy", new paco_dummy_fabrique());
    cerr << "[client] thread lib step"  << endl;  
#ifndef PADICO
    pfm->register_thread("omni", new paco_omni_fabrique());
#else
    pfm->register_thread("marcel", new paco_marcel_fabrique());
#endif

    // global context
    PaCO_operation * global_ptr = servant->getContext("global_paco_context");
    global_ptr->setLibCom("dummy",NULL);
#ifndef PADICO
    global_ptr->setLibThread("omni");
#else
    global_ptr->setLibThread("marcel");
#endif
    
    // work_in context
    PaCO_operation * method_ptr = servant->getContext("work_in");
    method_ptr->setDisLibArg(0,"GaBro","in");
    GaBro * dislib = (GaBro *) method_ptr->libraries_in[0];
    dislib->setEltSize(sizeof(long));

    obj = servant->_this();
    char buffer [5];
    snprintf(buffer,5,"%d",myid);
    string nom = "Servant";
    nom = nom + buffer;
    CosNaming::Name * name = nc->to_name(nom.c_str());
    nc->rebind(*name,obj);
    cerr << "[Server] Servant name : " << nom << endl;

    cerr << "[Server] Waiting requests" << endl;
    pman->activate();

#ifndef PADICO
    orb->run();
#else
    {
        marcel_mutex_t m;
        marcel_mutex_init(&m,0);
        marcel_mutex_lock(&m);
        marcel_mutex_lock(&m);
    }
#endif

    cerr << "[Server] server end" << endl;
    orb->destroy();
}
