#include<string>
#ifndef PADICO
#include <CORBA.h>
#include <Naming.hh>
#include <thread/paco_omni.h>
#else
#include <Padico/CORBA.h>
#include <Padico/COSNaming.h>
#include <thread/paco_marcel.h>
#endif
#include <interfacePaCO.h>
#include <interfacePaCO_Basic_Example_client.h>
#include <com/paco_mpi.h>
#include <stdio.h>
#include <GaBro.h>
#include <comScheduling/paco_libgraph_comScheduling.h>
#include <comScheduling/paco_direct_comScheduling.h>
#include <mpi.h>

static int myid;
static int numprocs;

static Basic::PaCO_Example * serveur;


// Loading librairies into the fabrics
void init() 
{
  paco_fabrique_manager* pfm = paco_getFabriqueManager();
  cerr << "[client] distributed lib step"  << endl;
  pfm->register_distribution("GaBro", new GaBro_fab());
  cerr << "[client] communication lib step"  << endl;
  pfm->register_com("mpi", new paco_mpi_fabrique());
  cerr << "[client] thread lib step"  << endl;  
#ifndef PADICO
  pfm->register_thread("mythreads", new paco_omni_fabrique());
#else
  pfm->register_thread("mythreads", new paco_marcel_fabrique());
#endif
  cerr << "[client] scheduling lib step"  << endl;
  pfm->register_comScheduling("libgraph", new paco_libgraph_fabrique());
  //pfm->register_comScheduling("direct", new paco_direct_fabrique());
}


void test() 
{

  // Init CORBA data
  cerr << "[client] data creation" << endl;
  int len1 = 10;
  int len2 = 5;

  Basic::Example::lseq data1;
  data1.length(len1);
  for (unsigned i=0; i< len1; i++) 
    {
      data1[i] = i + i;
    }
  
  Basic::Example::lseq data2;
  data2.length(len2);
  for (unsigned i=0; i< len2; i++) 
    {
      data2[i] = i * i;
    }
  
  // Init PaCO++ layers
  
  // global context
  cerr << "[client] init global context" << endl;
  PaCO_operation * global_ptr = serveur->getContext("global_paco_context");

  MPI_Comm group = MPI_COMM_WORLD;
  global_ptr->setLibCom("mpi",&group);

  global_ptr->setLibThread("mythreads");
  global_ptr->setTypeClient(true);
  PaCO::PacoTopology_t topo;
  topo.total = numprocs;
  global_ptr->setClientTopo(topo);
  
  // work_in context
  cerr << "[client] init work_in context" << endl;
  PaCO_operation * method_ptr = serveur->getContext("work_in");
  method_ptr->setDisLibArg(0,"GaBro","in");
  GaBro * dislib = (GaBro *) method_ptr->libraries_in[0];

  
  // Algorille
  cerr << "[client] init algorille lib" << endl;
  global_ptr->setLibComScheduling("libgraph"); 
//  global_ptr->setLibComScheduling("direct"); 
   paco_comScheduling_libgraph* libgraph_ptr = 
 	    static_cast<paco_comScheduling_libgraph*>(global_ptr->my_comScheduling);

   libgraph_ptr->set_beta(1);
   libgraph_ptr->set_k(numprocs);

  // End init PaCO++ layers
  cerr << "[client] final init" << endl;
  global_ptr->init(myid, numprocs);

  // data1
  dislib->setEltSize(sizeof(long));
  dislib->setNodeNbElt(len1);
  dislib->setTotalNbElt(numprocs*len1);
  dislib->setNodePos(myid*len1);
  method_ptr->useCommunicationMemoryId(1);

  // Send data
  cerr << "[client] sending data1" << endl;
  serveur->work_in(data1);
  
  // data2
  dislib->setNodeNbElt(len2);
  dislib->setTotalNbElt(numprocs*len2);
  dislib->setNodePos(myid*len2);
  method_ptr->useCommunicationMemoryId(2);

  // Send data
  cerr << "[client] sending data2" << endl;
  serveur->work_in(data2);

  // data1
  method_ptr->useCommunicationMemoryId(1);

  // Send data
  cerr << "[client] sending data1" << endl;
  serveur->work_in(data1);
}

int main(int argc, char **argv)
{
    cerr << "[Client] MPI Init" << endl;
    MPI_Init(&argc, &argv);
    MPI_Comm_size(MPI_COMM_WORLD,&numprocs);
    MPI_Comm_rank(MPI_COMM_WORLD,&myid);

    // CORBA Init part
    cerr << "[Client] CORBA init" << endl;
#ifndef PADICO
    CORBA::ORB_var orb = CORBA::ORB_init(argc, argv);
#else
    Padico::ORB_ptr _orb = Padico::ORB_init();
    CORBA::ORB_ptr orb = Padico::the_orb;
#endif
    CORBA::Object_var obj = orb->resolve_initial_references("NameService");
    CosNaming::NamingContextExt_var nc =
        CosNaming::NamingContextExt::_narrow(obj);
    cerr << "[client] get the server" << endl;
    obj = nc->resolve_str("ParallelServeur");
    cerr << "[client] PaCO_narrow !!!" << endl;
    serveur = (Basic::PaCO_Example *) Basic::PaCO_Example::PaCO_narrow(obj, orb);

    init();

    // Test
    test();

    // End
    MPI_Finalize();
#ifndef PADICO
    orb->destroy();
#endif
}
