#include <mpi.h>
#ifndef PADICO
#include <CORBA.h>
#include <Naming.hh>
#include <thread/paco_omni.h>
#else
#include <Padico/CORBA.h>
#include <Padico/COSNaming.h>
#include <thread/paco_marcel.h>
#endif
#include <iostream.h>
#include <servant_impl.h>
#include <stdio.h>
#include <com/paco_mpi.h>

#include <GaBro.h>

int main(int argc, char **argv)
{

    int  myid, numprocs;

    // Init
    cerr<< "[Serveur] Initialisation du serveur" << endl;

    // Init de MPI
    cerr << "[Serveur] Initialisation de MPI" << endl;
    MPI_Init(&argc, &argv);
    MPI_Comm_size(MPI_COMM_WORLD,&numprocs);
    MPI_Comm_rank(MPI_COMM_WORLD,&myid);
    cerr << "[Serveur] Fin initialisation de MPI" << endl;

    // Init de CORBA
#ifndef PADICO
    CORBA::ORB_ptr orb = CORBA::ORB_init(argc, argv);
#else
    CORBA::ORB_ptr orb = Padico::the_orb;
#endif

    CORBA::Object_var obj = orb->resolve_initial_references("NameService");
    CosNaming::NamingContextExt_var nc = CosNaming::NamingContextExt::_narrow(obj);
    obj = orb->resolve_initial_references("RootPOA");
    PortableServer::POA_var root_poa = PortableServer::POA::_narrow(obj);
    PortableServer::POAManager_var pman = root_poa->the_POAManager();

    // La bib de distribution ne supporte que le modele mono_thread
    CORBA::PolicyList pl;
    pl.length(1);
    pl[0] = root_poa->create_thread_policy (PortableServer::SINGLE_THREAD_MODEL);
    PortableServer::POA_var poa = root_poa->create_POA ("MyPOA", pman, pl);

    // Recuperation du proxy
    cerr << "[Serveur] find proxy" << endl;
    obj = nc->resolve_str("ParallelServeur");

    // Creation du noeud
    cerr << "[Serveur] Creating servant" << endl;
    char * ior = orb->object_to_string(obj);
    example_serv_impl * servant = new example_serv_impl(orb,ior);
    
    paco_fabrique_manager * pfm = paco_getFabriqueManager();
    pfm->register_distribution("GaBro", new GaBro_fab());
    cerr << "[client] communication lib step"  << endl;
    pfm->register_com("mpi", new paco_mpi_fabrique());
    cerr << "[client] thread lib step"  << endl;  
#ifndef PADICO
    pfm->register_thread("omni", new paco_omni_fabrique());
#else
    pfm->register_thread("marcel", new paco_marcel_fabrique());
#endif

    // Contexte global ...
    PaCO_operation * global_ptr = servant->getContext("global_paco_context");
    MPI_Comm group = MPI_COMM_WORLD;
    global_ptr->setLibCom("mpi",&group);
#ifndef PADICO
    global_ptr->setLibThread("omni");
#else
    global_ptr->setLibThread("marcel");
#endif
    
    // Contexte de method ...
    PaCO_operation * method_ptr = servant->getContext("work_in");
    method_ptr->setDisLibArg(0,"GaBro","in");
    GaBro * dislib = (GaBro *) method_ptr->libraries_in[0];
    dislib->setEltSize(sizeof(long));

    obj = servant->_this();
    char buffer [5];
    snprintf(buffer,5,"%d",myid);
    string nom = "Servant";
    nom = nom + buffer;
    CosNaming::Name * name = nc->to_name(nom.c_str());
    nc->rebind(*name,obj);
    cout << "Le nom du servant est : " << nom << endl;
    cerr << "[Serveur] Activating servant" << endl;
    PortableServer::ObjectId_var servantid = poa->activate_object(servant);
    // Fin Init
    cerr << "[Serveur] Waiting requests" << endl;
    pman->activate();

#ifndef PADICO
    orb->run();
#else
    {
        marcel_mutex_t m;
        marcel_mutex_init(&m,0);
        marcel_mutex_lock(&m);
        marcel_mutex_lock(&m);
    }
#endif

    // Fin
    cerr << "[Serveur] Fin du serveur" << endl;
    orb->destroy();
    MPI_Finalize();
}
